# -*- coding: utf-8 -*-
"""
Test switching and printing the current database.
"""

import pytest
from pymssql import _mssql

from .helpers import config

@pytest.mark.mssql_server_required
@pytest.mark.parametrize("db", [
    config.database,
    'master',
    'tempdb',
    'msdb',
    'model',
])
def test_connect_cur_db_name(db):
    conn = _mssql.connect(
        server = config.server,
        port = config.port,
        user = config.user,
        password = config.password,
        database = db,
    )
    connDBName = conn.execute_scalar("SELECT DB_NAME() AS myDB")
    assert db == connDBName
    assert conn.cur_db_name() == connDBName
    conn.close()

@pytest.mark.mssql_server_required
def test_select_db():
    conn = _mssql.connect(
        server = config.server,
        port = config.port,
        user = config.user,
        password = config.password,
        database = config.database,
    )
    for db in ['master', 'model', 'msdb', 'tempdb']:
        conn.select_db(db)
        connDBName = conn.execute_scalar("SELECT DB_NAME() AS myDB")
        assert connDBName == db
        assert conn.cur_db_name() == db
    conn.close()



"""
Test maximum query size.
"""

import unittest

import pymssql

import pytest
from .helpers import pymssqlconn


@pytest.mark.xfail(strict=False, reason="Could timeout, or fail with different error messages")
@pytest.mark.timeout(60)
@pytest.mark.mssql_server_required
class QueryTests(unittest.TestCase):

    def setUp(self):
        self.connection = pymssqlconn()
        self.connection._conn.execute_non_query("""CREATE TABLE pymssql (data text)""")


    def tearDown(self):
        self.connection._conn.execute_non_query('DROP TABLE pymssql')
        self.connection.close()


    def test_query_size(self):

        cur = self.connection.cursor()
        cur.execute('DELETE FROM pymssql')
        query = "INSERT INTO pymssql (data) VALUES (%s)"
        low = 1024*1024 #*127
        high = 1024*1024*128
        n = 0
        print("")
        while 1:
            n += 1
            next = (low + high) // 2
            cur.execute("DELETE FROM pymssql")
            value = "A"*next
            try:
                cur.execute(query, value)
                cur.execute("SELECT * FROM pymssql")
                res = cur.fetchone()[0]
                if res==value:
                    low = next
                else:
                    print("\t\tres != value")
                    high = next
            except pymssql.OperationalError:
                high = next
                next = low
                self.connection = pymssqlconn()
                self.connection._conn.execute_non_query("""CREATE TABLE pymssql (data text)""")
                cur = self.connection.cursor()

            if high - low == 1:
                break

        print(f"Found max size = {next:#x} {next / (1024*1024)} MiB")
        self.assertTrue(next > self.connection._conn.max_query_size)
